from djangoldp.permissions import LDPPermissions
from django.urls import resolve
from djangoldp.utils import is_authenticated_user
from djangoldp_project.permissions import ProjectPermissions
from djangoldp_circle.permissions import CirclePermissions


class CommunityPermissions(LDPPermissions):

    filter_backends = []

    def _get_community_from_obj(self, obj):
        from djangoldp_community.models import Community
        if isinstance(obj, Community):
            return obj

        if not hasattr(obj, 'community') or not isinstance(obj.community, Community):
            raise KeyError('Object ' + str(obj) + ' must have a ForeignKey "community" to model Community to use CommunityPermissions')

        return obj.community

    def get_object_permissions(self, request, view, obj):
        from djangoldp_community.models import Community, CommunityMember, CommunityProfile, CommunityAddress

        perms = set(super().get_object_permissions(request, view, obj))
        allow_continue = True
        if hasattr(obj, 'project'):
            if (is_authenticated_user(request.user) and not obj.project.members.filter(user=request.user).exists()) or obj.project.status != "Public":
                allow_continue = False
        elif hasattr(obj, 'circle'):
            if (is_authenticated_user(request.user) and not obj.circle.members.filter(user=request.user).exists()) or obj.circle.status != "Public":
                allow_continue = False

        if allow_continue:
            perms = perms.union({'view'})

            community = self._get_community_from_obj(obj)

            if is_authenticated_user(request.user) and community.members.filter(user=request.user).exists():
                # Any member can add a job offer, circle or project to the community
                if not isinstance(obj, Community) and not isinstance(obj, CommunityMember) and\
                   not isinstance(obj, CommunityProfile) and not isinstance(obj, CommunityAddress):
                    perms = perms.union({'add'})

                member = community.members.get(user=request.user)
                if member.is_admin:
                    # Admins can add members
                    perms = perms.union({'add', 'change'})

                    if isinstance(obj, CommunityMember):
                        if (obj.user == request.user and community.members.filter(is_admin=True).count() > 1) \
                                or not obj.is_admin:
                            # Admins can't delete community or other admins, but have super-powers on everything else
                            # I can't delete myself if I am the last member
                            perms = perms.union({'delete'})
                    elif not isinstance(obj, Community) and not isinstance(obj, CommunityProfile):
                        perms = perms.union({'delete'})
                elif isinstance(obj, CommunityMember) and obj.user == request.user:
                    perms = perms.union({'delete'})

        return perms


class CommunityCirclePermissions(CirclePermissions):
    filter_backends = []

    def get_object_permissions(self, request, view, obj):
        obj = obj.circle
        return set(super().get_object_permissions(request, view, obj))

    def get_container_permissions(self, request, view, obj=None):
        perms = super().get_container_permissions(request, view, obj, True)
        if obj is None:
            from djangoldp_community.models import Community
            resolved = resolve(request.path_info)
            if 'slug' in resolved.kwargs and (resolved.url_name == "community-communitycircle-list" or resolved.url_name == "community-detail"):
                community = Community.objects.get(slug=resolved.kwargs['slug'])
                if is_authenticated_user(request.user) and community.members.filter(user=request.user).exists():
                    perms = perms.union({'add'})
        return perms


class CommunityProjectPermissions(ProjectPermissions):
    filter_backends = []

    def get_object_permissions(self, request, view, obj):
        obj = obj.project
        return set(super().get_object_permissions(request, view, obj))

    def get_container_permissions(self, request, view, obj=None):
        perms = super().get_container_permissions(request, view, obj, True)
        if obj is None:
            from djangoldp_community.models import Community
            resolved = resolve(request.path_info)
            if 'slug' in resolved.kwargs and (resolved.url_name == "community-communityproject-list" or resolved.url_name == "community-detail"):
                community = Community.objects.get(slug=resolved.kwargs['slug'])
                if is_authenticated_user(request.user) and community.members.filter(user=request.user).exists():
                    perms = perms.union({'add'})
        else:
            return self.get_object_permissions(request, view, obj)
        return perms


class CommunityJobPermissions(LDPPermissions):
    filter_backends = []

    def get_object_permissions(self, request, view, obj):
        obj = obj.joboffer
        return set(super().get_object_permissions(request, view, obj))

    def get_container_permissions(self, request, view, obj=None):
        perms = super().get_container_permissions(request, view, obj)
        if obj is None:
            from djangoldp_community.models import Community
            resolved = resolve(request.path_info)
            if 'slug' in resolved.kwargs and (resolved.url_name == "community-communityjoboffer-list" or resolved.url_name == "community-detail"):
                community = Community.objects.get(slug=resolved.kwargs['slug'])
                if is_authenticated_user(request.user) and community.members.filter(user=request.user).exists():
                    perms = perms.union({'add'})
        else:
            return self.get_object_permissions(request, view, obj)
        return perms


class CommunityMembersPermissions(CommunityPermissions):
    filter_backends = []

    def get_container_permissions(self, request, view, obj=None):
        perms = set({'view'})
        if obj is None:
            from djangoldp_community.models import Community
            resolved = resolve(request.path_info)
            if 'slug' in resolved.kwargs and (resolved.url_name == "community-communitymember-list" or resolved.url_name == "community-detail"):
                community = Community.objects.get(slug=resolved.kwargs['slug'])
                if is_authenticated_user(request.user) and community.members.filter(user=request.user).exists():
                    if community.members.get(user=request.user).is_admin:
                        perms = perms.union({'add'})
                if community.allow_self_registration:
                    perms = perms.union({'add'})
        else:
            return self.get_object_permissions(request, view, obj)
        return perms

class CommunityProfilePermissions(CommunityPermissions):
    filter_backends = []

    def get_container_permissions(self, request, view, obj=None):
        perms = set({'view'})
        if obj is None:
            from djangoldp_community.models import Community
            resolved = resolve(request.path_info)
            if 'slug' in resolved.kwargs and (resolved.url_name == "community-communityprofile-list" or\
                resolved.url_name == "community-communityprofile-detail" or resolved.url_name == "community-detail")\
                or resolved.url_name == "community-communityaddress-list":
                community = Community.objects.get(slug=resolved.kwargs['slug'])
                if is_authenticated_user(request.user) and community.members.filter(user=request.user).exists():
                    if community.members.get(user=request.user).is_admin:
                        perms = perms.union({'add', 'change'})
        else:
            return self.get_object_permissions(request, view, obj)
        return perms


from django.contrib import admin
from djangoldp.admin import DjangoLDPAdmin
from djangoldp.models import Model
from djangoldp_community.models import Community, CommunityMember, CommunityCircle,\
    CommunityProject, CommunityJobOffer, CommunityProfile, CommunityAddress


class EmptyAdmin(admin.ModelAdmin):
    def get_model_perms(self, request):
        return {}


class MemberInline(admin.TabularInline):
    model = CommunityMember
    exclude = ('urlid', 'is_backlink', 'allow_create_backlink')
    extra = 0


class CircleInline(admin.TabularInline):
    model = CommunityCircle
    exclude = ('urlid', 'is_backlink', 'allow_create_backlink')
    extra = 0


class ProjectInline(admin.TabularInline):
    model = CommunityProject
    exclude = ('urlid', 'is_backlink', 'allow_create_backlink')
    extra = 0


class JobOfferInline(admin.TabularInline):
    model = CommunityJobOffer
    exclude = ('urlid', 'is_backlink', 'allow_create_backlink')
    extra = 0


class ProfileInline(admin.StackedInline):
    model = CommunityProfile
    exclude = ('urlid', 'is_backlink', 'allow_create_backlink')
    extra = 0


class AddressInline(admin.TabularInline):
    model = CommunityAddress
    exclude = ('urlid', 'is_backlink', 'allow_create_backlink')
    extra = 0


class CommunityAdmin(DjangoLDPAdmin):
    list_display = ('urlid', 'name', 'allow_self_registration')
    exclude = ('urlid', 'slug', 'is_backlink', 'allow_create_backlink')
    inlines = [ProfileInline, AddressInline, MemberInline, CircleInline, ProjectInline, JobOfferInline]
    search_fields = ['urlid', 'name', 'members__user__urlid', 'circles__circle__name']
    ordering = ['urlid']

    def get_queryset(self, request):
        # Hide distant communities
        queryset = super(CommunityAdmin, self).get_queryset(request)
        internal_ids = [x.pk for x in queryset if not Model.is_external(x)]
        return queryset.filter(pk__in=internal_ids)


admin.site.register(Community, CommunityAdmin)
admin.site.register(CommunityMember, EmptyAdmin)
admin.site.register(CommunityCircle, EmptyAdmin)
admin.site.register(CommunityProject, EmptyAdmin)
admin.site.register(CommunityJobOffer, EmptyAdmin)
admin.site.register(CommunityProfile, EmptyAdmin)
admin.site.register(CommunityAddress, EmptyAdmin)



# Generated by Django 2.2.18 on 2021-04-15 16:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_community', '0007_auto_20210324_1146'),
    ]

    operations = [
        migrations.AlterField(
            model_name='community',
            name='name',
            field=models.CharField(blank=True, help_text="Changing a community's name is highly discouraged", max_length=255, null=True),
        ),
    ]



# Generated by Django 2.2.18 on 2021-03-24 09:54

from django.db import migrations, models
import django.db.models.deletion
import djangoldp.fields
import djangoldp_community.permissions


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_community', '0004_auto_20210323_1121'),
    ]

    operations = [
        migrations.CreateModel(
            name='CommunityProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('shortDescription', models.CharField(blank=True, default='', max_length=254, null=True)),
                ('description', models.TextField(blank=True, default='', null=True)),
                ('phone', models.CharField(blank=True, default='', max_length=254, null=True)),
                ('email', models.EmailField(blank=True, default='', max_length=254, null=True)),
                ('website', models.URLField(blank=True, default='', null=True)),
                ('tweeter', models.URLField(blank=True, default='', null=True)),
                ('facebook', models.URLField(blank=True, default='', null=True)),
                ('linkedin', models.URLField(blank=True, default='', null=True)),
                ('instagram', models.URLField(blank=True, default='', null=True)),
                ('picture1', models.URLField(blank=True, default='', null=True)),
                ('picture2', models.URLField(blank=True, default='', null=True)),
                ('picture3', models.URLField(blank=True, default='', null=True)),
                ('community', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='profile', to='djangoldp_community.Community')),
            ],
            options={
                'verbose_name': 'community profile',
                'verbose_name_plural': 'communitie profiles',
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:CommunityProfile',
                'container_path': 'community-profiles/',
                'permission_classes': [djangoldp_community.permissions.CommunityProfilePermissions],
                'serializer_fields': ['@id', 'community', 'shortDescription', 'description', 'phone', 'email', 'website', 'tweeterfacebook', 'linkedin', 'instagram', 'picture1', 'picture2', 'picture3'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit'],
                'superuser_perms': ['view'],
            },
        ),
        migrations.CreateModel(
            name='CommunityAddress',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('address', models.CharField(blank=True, default='', max_length=254, null=True)),
                ('lat', models.DecimalField(blank=True, decimal_places=12, max_digits=15, null=True, verbose_name='Latitude')),
                ('lon', models.DecimalField(blank=True, decimal_places=12, max_digits=15, null=True, verbose_name='Longitude')),
                ('community', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='addresses', to='djangoldp_community.Community')),
            ],
            options={
                'verbose_name': 'community address',
                'verbose_name_plural': 'communitie addresses',
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:CommunityAddress',
                'container_path': 'community-addresses/',
                'permission_classes': [djangoldp_community.permissions.CommunityProfilePermissions],
                'serializer_fields': ['@id', 'community', 'address', 'lat', 'lon'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit'],
                'superuser_perms': ['view'],
            },
        ),
    ]



# Generated by Django 2.2.18 on 2021-03-23 10:21

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_community', '0003_auto_20210218_1145'),
    ]

    operations = [
        migrations.AlterField(
            model_name='communitycircle',
            name='circle',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='community', to='djangoldp_circle.Circle'),
        ),
        migrations.AlterField(
            model_name='communityjoboffer',
            name='joboffer',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='community', to='djangoldp_joboffer.JobOffer'),
        ),
        migrations.AlterField(
            model_name='communityproject',
            name='project',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='community', to='djangoldp_project.Project'),
        ),
    ]



import uuid
from django.contrib.auth import get_user_model
from django.conf import settings
from django.db import models
from django.db.models.signals import pre_save, post_save
from django.dispatch import receiver
from djangoldp.models import Model
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _

from djangoldp_community.permissions import CommunityPermissions, CommunityCirclePermissions, \
    CommunityProjectPermissions, CommunityJobPermissions, CommunityMembersPermissions, \
    CommunityProfilePermissions
from djangoldp_community.views import CommunityMembersViewset

from djangoldp_circle.models import Circle
from djangoldp_project.models import Project
from djangoldp_joboffer.models import JobOffer
from rest_framework.exceptions import ValidationError

class Community(Model):
    name = models.CharField(max_length=255, blank=True, null=True, help_text="Changing a community's name is highly discouraged")
    logo = models.URLField(blank=True, null=True)
    allow_self_registration = models.BooleanField(default=False)
    slug = models.SlugField(unique=True, blank=True, null=True)

    def __str__(self):
        try:
            return '{} ({})'.format(self.name, self.urlid)
        except:
            return self.urlid

    class Meta(Model.Meta):
        verbose_name = _('community')
        verbose_name_plural = _("communities")
        permission_classes = [CommunityPermissions]
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add']
        superuser_perms = ['view']
        lookup_field = 'slug'
        container_path = "/communities/"
        ordering = ['slug']
        serializer_fields = ['@id', 'name', 'profile', 'addresses', 'logo', 'allow_self_registration',\
                             'projects', 'circles', 'members', 'joboffers']
        rdf_type = "sib:Community"
        depth = 1

class CommunityProfile(Model):
    community = models.OneToOneField(Community, on_delete=models.CASCADE, related_name='profile', null=True, blank=True)
    shortDescription = models.CharField(max_length=254, blank=True, null=True, default='')
    description = models.TextField(blank=True, null=True, default='')
    phone = models.CharField(max_length=254, blank=True, null=True, default='')
    email = models.EmailField(max_length=254, blank=True, null=True, default='')
    website = models.URLField(blank=True, null=True, default='')
    tweeter = models.URLField(blank=True, null=True, default='')
    facebook = models.URLField(blank=True, null=True, default='')
    linkedin = models.URLField(blank=True, null=True, default='')
    instagram = models.URLField(blank=True, null=True, default='')
    picture1 = models.URLField(blank=True, null=True, default='')
    picture2 = models.URLField(blank=True, null=True, default='')
    picture3 = models.URLField(blank=True, null=True, default='')

    def __str__(self):
        try:
            return '{} ({})'.format(self.community.urlid, self.urlid)
        except:
            return self.urlid

    class Meta(Model.Meta):
        verbose_name = _('community profile')
        verbose_name_plural = _("community profiles")
        permission_classes = [CommunityProfilePermissions]
        anonymous_perms = ['view']
        authenticated_perms = ['inherit']
        superuser_perms = ['view']
        container_path = "community-profiles/"
        serializer_fields = ['@id', 'community', 'shortDescription', 'description', 'phone', 'email', 'website', 'tweeter',\
                             'facebook', 'linkedin', 'instagram', 'picture1', 'picture2', 'picture3']
        rdf_type = "sib:CommunityProfile"

class CommunityAddress(Model):
    community = models.ForeignKey(Community, on_delete=models.CASCADE, related_name='addresses', null=True, blank=True)
    address_line1 = models.CharField(max_length=254, blank=True, null=True, default='')
    address_line2 = models.CharField(max_length=254, blank=True, null=True, default='')
    lat = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=_("Latitude"))
    lng = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=_("Longitude"))

    def __str__(self):
        try:
            return '{} ({})'.format(self.community.urlid, self.urlid)
        except:
            return self.urlid

    class Meta(Model.Meta):
        verbose_name = _('community address')
        verbose_name_plural = _("community addresses")
        permission_classes = [CommunityProfilePermissions]
        anonymous_perms = ['view']
        authenticated_perms = ['inherit']
        superuser_perms = ['view']
        container_path = "community-addresses/"
        serializer_fields = ['@id', 'community', 'address_line1', 'address_line2', 'lat', 'lng']
        rdf_type = "sib:CommunityAddress"

class CommunityMember(Model):
    community = models.ForeignKey(Community, on_delete=models.CASCADE, related_name='members', null=True, blank=True)
    user = models.ForeignKey(get_user_model(), on_delete=models.CASCADE, related_name="communities", null=True, blank=True)
    is_admin = models.BooleanField(default=False)

    def __str__(self):
        try:
            return '{} -> {} ({})'.format(self.user.urlid, self.community.urlid, self.urlid)
        except:
            return self.urlid

    class Meta(Model.Meta):
        verbose_name = _('community member')
        verbose_name_plural = _("community members")
        view_set = CommunityMembersViewset
        permission_classes = [CommunityMembersPermissions]
        anonymous_perms = ['view']
        authenticated_perms = ['inherit']
        superuser_perms = ['inherit']
        container_path = "community-members/"
        serializer_fields = ['@id', 'community', 'user', 'is_admin']
        rdf_type = "as:items"

class CommunityCircle(Model):
    community = models.ForeignKey(Community, on_delete=models.CASCADE, related_name='circles', null=True, blank=True)
    circle = models.OneToOneField(Circle, on_delete=models.CASCADE, related_name="community", null=True, blank=True)

    def __str__(self):
        try:
            return '{} -> {} ({})'.format(self.circle.urlid, self.community.urlid, self.urlid)
        except:
            return self.urlid

    class Meta(Model.Meta):
        verbose_name = _('community circle')
        verbose_name_plural = _("community circles")
        permission_classes = [CommunityPermissions, CommunityCirclePermissions]
        anonymous_perms = ['view']
        authenticated_perms = ['inherit']
        superuser_perms = ['inherit']
        container_path = "community-circles/"
        serializer_fields = ['@id', 'community', 'circle']
        rdf_type = "as:items"

class CommunityProject(Model):
    community = models.ForeignKey(Community, on_delete=models.CASCADE, related_name='projects', null=True, blank=True)
    project = models.OneToOneField(Project, on_delete=models.CASCADE, related_name="community", null=True, blank=True)

    def __str__(self):
        try:
            return '{} -> {} ({})'.format(self.project.urlid, self.community.urlid, self.urlid)
        except:
            return self.urlid

    class Meta(Model.Meta):
        verbose_name = _('community project')
        verbose_name_plural = _("community projects")
        permission_classes = [CommunityPermissions, CommunityProjectPermissions]
        anonymous_perms = ['view']
        authenticated_perms = ['inherit']
        superuser_perms = ['inherit']
        container_path = "community-projects/"
        serializer_fields = ['@id', 'community', 'project']
        rdf_type = "as:items"

class CommunityJobOffer(Model):
    community = models.ForeignKey(Community, on_delete=models.CASCADE, related_name='joboffers', null=True, blank=True)
    joboffer = models.OneToOneField(JobOffer, on_delete=models.CASCADE, related_name="community", null=True, blank=True)

    def __str__(self):
        try:
            return '{} -> {} ({})'.format(self.joboffer.urlid, self.community.urlid, self.urlid)
        except:
            return self.urlid

    class Meta(Model.Meta):
        verbose_name = _('community job offer')
        verbose_name_plural = _("community job offers")
        permission_classes = [CommunityPermissions, CommunityJobPermissions]
        anonymous_perms = ['view']
        authenticated_perms = ['inherit']
        superuser_perms = ['inherit']
        container_path = "community-joboffers/"
        serializer_fields = ['@id', 'community', 'joboffer']
        rdf_type = "as:items"

@receiver(pre_save, sender=Community)
def pre_create_account(sender, instance, **kwargs):
    if not instance.urlid or instance.urlid.startswith(settings.SITE_URL):
        if getattr(instance, Model.slug_field(instance)) != slugify(instance.name):
            if Community.objects.local().filter(slug=slugify(instance.name)).count() > 0:
                raise ValidationError(_("Community name must be unique"))
            setattr(instance, Model.slug_field(instance), slugify(instance.name))
            setattr(instance, "urlid", "")
    else:
        # Is a distant object, generate a random slug
        setattr(instance, Model.slug_field(instance), uuid.uuid4().hex.upper()[0:8])

@receiver(post_save, sender=Community)
def create_community_profile(instance, created, **kwargs):
    if not Model.is_external(instance):
        CommunityProfile.objects.get_or_create(community=instance)



# Generated by Django 2.2.18 on 2021-03-24 10:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_community', '0006_auto_20210324_1113'),
    ]

    operations = [
        migrations.RenameField(
            model_name='communityaddress',
            old_name='address',
            new_name='address_line1',
        ),
        migrations.AddField(
            model_name='communityaddress',
            name='address_line2',
            field=models.CharField(blank=True, default='', max_length=254, null=True),
        ),
    ]



# Generated by Django 2.2.16 on 2020-10-13 16:02

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import djangoldp.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('djangoldp_circle', '0023_auto_20200617_1817'),
        ('djangoldp_joboffer', '0007_auto_20200924_1937'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('djangoldp_project', '0014_auto_20200903_1051'),
    ]

    operations = [
        migrations.CreateModel(
            name='Community',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('name', models.CharField(blank=True, max_length=255)),
                ('slug', models.SlugField(blank=True, null=True, unique=True)),
            ],
            options={
                'verbose_name': 'community',
                'verbose_name_plural': 'communities',
                'ordering': ['slug'],
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'control'),
            },
        ),
        migrations.CreateModel(
            name='CommunityProject',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('community', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='projects', to='djangoldp_community.Community')),
                ('project', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='communities', to='djangoldp_project.Project')),
            ],
            options={
                'verbose_name': 'community project',
                'verbose_name_plural': 'community projects',
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'control'),
            },
        ),
        migrations.CreateModel(
            name='CommunityMember',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('is_admin', models.BooleanField(default=False)),
                ('community', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='members', to='djangoldp_community.Community')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='communities', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'community member',
                'verbose_name_plural': 'community members',
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'control'),
            },
        ),
        migrations.CreateModel(
            name='CommunityJobOffer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('community', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='joboffers', to='djangoldp_community.Community')),
                ('joboffer', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='communities', to='djangoldp_joboffer.JobOffer')),
            ],
            options={
                'verbose_name': 'community job offer',
                'verbose_name_plural': 'community job offers',
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'control'),
            },
        ),
        migrations.CreateModel(
            name='CommunityCircle',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('circle', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='communities', to='djangoldp_circle.Circle')),
                ('community', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='circles', to='djangoldp_community.Community')),
            ],
            options={
                'verbose_name': 'community circle',
                'verbose_name_plural': 'community circles',
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'control'),
            },
        ),
    ]



from django.http import Http404
from djangoldp.views import LDPViewSet
from djangoldp.utils import is_authenticated_user


class CommunityMembersViewset(LDPViewSet):

    def get_parent(self):
        raise NotImplementedError("get_parent not implemented in CommunityMembersViewSet")

    def is_safe_create(self, user, validated_data, *args, **kwargs):
        from djangoldp_community.models import Community

        try:
            if 'community' in validated_data.keys():
                community = Community.objects.get(urlid=validated_data['community']['urlid'])
            else:
                community = self.get_parent()

            if community.allow_self_registration or \
                    (is_authenticated_user(user) and community.members.filter(user=user, is_admin=True).exists()):
                return True
        except Community.DoesNotExist:
            return True
        except (KeyError, AttributeError):
            raise Http404('community not specified with urlid')

        return False


class OpenCommunitiesViewset(LDPViewSet):
  def get_queryset(self):
    queryset = super().get_queryset().exclude(allow_self_registration=False)
    # invalidate cache for every open communities, unless that if /open-communities/ is loaded before /communities/xyz/, the last one will get wrong permission nodes
    from djangoldp.serializers import LDListMixin, LDPSerializer
    LDListMixin.to_representation_cache.reset()
    for result in queryset:
      if(result.urlid):
        LDPSerializer.to_representation_cache.invalidate(result.urlid)
    return queryset



# Generated by Django 2.2.18 on 2021-02-17 11:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_community', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='community',
            name='allow_self_registration',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='community',
            name='logo',
            field=models.URLField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.18 on 2021-03-24 10:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_community', '0005_communityaddress_communityprofile'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='communityaddress',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control'], 'verbose_name': 'community address', 'verbose_name_plural': 'community addresses'},
        ),
        migrations.AlterModelOptions(
            name='communityprofile',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control'], 'verbose_name': 'community profile', 'verbose_name_plural': 'community profiles'},
        ),
        migrations.RenameField(
            model_name='communityaddress',
            old_name='lon',
            new_name='lng',
        ),
    ]



from djangoldp.serializers import LDListMixin, LDPSerializer
from rest_framework.exceptions import ValidationError
from rest_framework.test import APITestCase, APIClient

from djangoldp_community.models import Community


class ModelTestCase(APITestCase):
    def setUp(self):
        self.client = APIClient()
        LDListMixin.to_representation_cache.reset()
        LDPSerializer.to_representation_cache.reset()

    # I should not be able to create duplicate names in local communities
    def test_cant_create_duplicate_local_communities(self):
        def create_community():
            Community.objects.create(name='Test')

        create_community()
        try:
            create_community()
            self.fail('Was able to create a local community with a duplicate name!')
        except ValidationError:
            pass

    # I should be able to create multiple backlinked communities without a name
    def test_can_create_duplicate_external_communities(self):
        Community.objects.create(urlid="https://example.com/external/1/")
        try:
            Community.objects.create(urlid="https://example.com/external/2/")
        except ValidationError as e:
            self.fail('Creating duplicate external Community raised exception ' + str(e))



import uuid
import json
from datetime import datetime, timedelta

from djangoldp.serializers import LDListMixin, LDPSerializer
from rest_framework.test import APITestCase, APIClient

from djangoldp_community.models import Community, CommunityMember, CommunityCircle, CommunityProject, CommunityJobOffer
from djangoldp_community.tests.models import User


class PermissionsTestCase(APITestCase):
    # Django runs setUp automatically before every test
    def setUp(self):
        # we set up a client, that allows us
        self.client = APIClient()
        LDListMixin.to_representation_cache.reset()
        LDPSerializer.to_representation_cache.reset()

    # we have custom set up functions for things that we don't want to run before *every* test, e.g. often we want to
    # set up an authenticated user, but sometimes we want to run a test with an anonymous user
    def setUpLoggedInUser(self, is_superuser=False):
        self.user = User(email='test@mactest.co.uk', first_name='Test', last_name='Mactest', username='test',
                         password='glass onion', is_superuser=is_superuser)
        self.user.save()
        # this means that our user is now logged in (as if they had typed username and password)
        self.client.force_authenticate(user=self.user)

    # we write functions like this for convenience - we can reuse between tests
    def _get_random_user(self):
        return User.objects.create(email='{}@test.co.uk'.format(str(uuid.uuid4())), first_name='Test',
                                   last_name='Test',
                                   username=str(uuid.uuid4()))

    def _get_random_community(self):
        return Community.objects.create(name='Test', slug=str(uuid.uuid4()))

    def _get_community_member(self, user, community, is_admin=False):
        return CommunityMember.objects.create(user=user, community=community, is_admin=is_admin)

    '''
    list communities - public
    list community members - public
    create community - authenticated
    update, delete, control community - community admin only
    create, update, delete, control community member - community admin only
    Admins can't remove admins (or themselves if they're the last admin)
    community projects - apply Project permissions (same for JobOffers and Circles)
    '''
    # only authenticated users can create communities
    def test_post_community_anonymous(self):
        response = self.client.post('/communities/', data=json.dumps({}), content_type='application/ld+json')
        self.assertEqual(response.status_code, 403)

    def test_post_community_authenticated(self):
        self.setUpLoggedInUser()
        response = self.client.post('/communities/', data=json.dumps({}), content_type='application/ld+json')
        self.assertEqual(response.status_code, 201)

    # only community admins can update communities
    def test_update_community_is_admin(self):
        self.setUpLoggedInUser()
        community = self._get_random_community()
        self._get_community_member(user=self.user, community=community, is_admin=True)
        response = self.client.patch('/communities/{}/'.format(community.slug), data=json.dumps({}),
                                     content_type='application/ld+json')
        self.assertEqual(response.status_code, 200)

    def test_update_community_is_member(self):
        self.setUpLoggedInUser(is_superuser=False)
        community = self._get_random_community()
        self._get_community_member(user=self.user, community=community, is_admin=False)
        response = self.client.patch('/communities/{}/'.format(community.slug), data=json.dumps({}),
                                     content_type='application/ld+json')
        self.assertEqual(response.status_code, 403)

    def test_update_community_is_auth_super_user(self):
        self.setUpLoggedInUser(is_superuser=True)
        community = self._get_random_community()
        self._get_community_member(user=self.user, community=community, is_admin=False)
        response = self.client.patch('/communities/{}/'.format(community.slug), data=json.dumps({}),
                                     content_type='application/ld+json')
        self.assertEqual(response.status_code, 403)

    # only community admins can delete communities
    def test_delete_community_is_admin(self):
        self.setUpLoggedInUser(is_superuser=False)
        community = self._get_random_community()
        self._get_community_member(user=self.user, community=community, is_admin=True)
        response = self.client.delete('/communities/{}/'.format(community.slug))
        self.assertEqual(response.status_code, 403)

    def test_delete_community_is_member(self):
        self.setUpLoggedInUser(is_superuser=False)
        community = self._get_random_community()
        self._get_community_member(user=self.user, community=community, is_admin=False)
        response = self.client.delete('/communities/{}/'.format(community.slug))
        self.assertEqual(response.status_code, 403)

    def test_delete_community_is_auth_super_user(self):
        self.setUpLoggedInUser(is_superuser=True)
        community = self._get_random_community()
        self._get_community_member(user=self.user, community=community, is_admin=False)
        response = self.client.delete('/communities/{}/'.format(community.slug))
        self.assertEqual(response.status_code, 403)

    # TODO: https://git.startinblox.com/djangoldp-packages/djangoldp/issues/363
    '''
    def test_get_community_is_member(self):
        self.setUpLoggedInUser(is_superuser=False)
        community = self._get_random_community()
        me = self._get_community_member(user=self.user, community=community, is_admin=False)

        response = self.client.get('/communities/{}/members/'.format(community.slug))
        self.assertEqual(response.status_code, 200)
        self.assertNotIn({'mode': {'@type': 'add'}}, response.data['permissions'])
        self.assertEqual(len(response.data['permissions']), 1)
    '''

    # only community admins can do any operation on community members
    def test_add_community_member_is_admin(self):
        self.setUpLoggedInUser(is_superuser=False)
        another_user = self._get_random_user()

        community = self._get_random_community()
        me = self._get_community_member(user=self.user, community=community, is_admin=True)

        body = {
            'http://happy-dev.fr/owl#community': community.urlid,
            'http://happy-dev.fr/owl#user': another_user.urlid
        }

        response = self.client.post('/communities/{}/members/'.format(community.slug),
                                    body=json.dumps(body), content_type='application/ld+json')
        self.assertEqual(response.status_code, 201)

    def test_add_community_member_is_member(self):
        self.setUpLoggedInUser(is_superuser=False)
        another_user = self._get_random_user()

        community = self._get_random_community()
        me = self._get_community_member(user=self.user, community=community, is_admin=False)

        body = {
            'http://happy-dev.fr/owl#community': community.urlid,
            'http://happy-dev.fr/owl#user': another_user.urlid
        }

        response = self.client.post('/communities/{}/members/', body=json.dumps(body), content_type='application/ld+json')
        self.assertEqual(response.status_code, 404)

    # TODO: https://git.startinblox.com/djangoldp-packages/djangoldp-community/issues/3
    '''
    def test_add_community_member_is_admin_no_parent(self):
        self.setUpLoggedInUser(is_superuser=False)
        another_user = self._get_random_user()

        community = self._get_random_community()
        me = self._get_community_member(user=self.user, community=community, is_admin=True)

        body = {
            'http://happy-dev.fr/owl#community': community.urlid,
            'http://happy-dev.fr/owl#user': another_user.urlid
        }

        response = self.client.post('/community-members/', body=json.dumps(body), content_type='application/ld+json')
        self.assertEqual(response.status_code, 201)
    '''

    def test_delete_community_member_is_admin(self):
        self.setUpLoggedInUser(is_superuser=False)
        another_user = self._get_random_user()

        community = self._get_random_community()
        me = self._get_community_member(user=self.user, community=community, is_admin=True)
        member = self._get_community_member(user=another_user, community=community, is_admin=False)

        response = self.client.delete('/community-members/{}/'.format(member.pk))
        self.assertEqual(response.status_code, 204)

    def test_delete_community_member_is_member(self):
        self.setUpLoggedInUser(is_superuser=False)
        another_user = self._get_random_user()

        community = self._get_random_community()
        me = self._get_community_member(user=self.user, community=community, is_admin=False)
        member = self._get_community_member(user=another_user, community=community, is_admin=False)

        response = self.client.delete('/community-members/{}/'.format(member.pk))
        self.assertEqual(response.status_code, 403)

    def test_delete_community_member_is_auth_super_user(self):
        self.setUpLoggedInUser(is_superuser=True)
        another_user = self._get_random_user()

        community = self._get_random_community()
        me = self._get_community_member(user=self.user, community=community, is_admin=False)
        member = self._get_community_member(user=another_user, community=community, is_admin=False)

        response = self.client.delete('/community-members/{}/'.format(member.pk))
        self.assertEqual(response.status_code, 403)

    # community admins cannot remove other admins
    def test_delete_community_admin_is_admin(self):
        self.setUpLoggedInUser(is_superuser=False)
        another_user = self._get_random_user()

        community = self._get_random_community()
        me = self._get_community_member(user=self.user, community=community, is_admin=True)
        member = self._get_community_member(user=another_user, community=community, is_admin=True)

        response = self.client.delete('/community-members/{}/'.format(member.pk))
        self.assertEqual(response.status_code, 403)

    # community admins can remove themselves
    def test_delete_self_is_admin(self):
        self.setUpLoggedInUser(is_superuser=False)
        another_user = self._get_random_user()

        community = self._get_random_community()
        me = self._get_community_member(user=self.user, community=community, is_admin=True)
        member = self._get_community_member(user=another_user, community=community, is_admin=True)

        response = self.client.delete('/community-members/{}/'.format(me.pk))
        self.assertEqual(response.status_code, 204)

    # community admins cannot remove themselves if they are the last admin
    def test_delete_self_is_last_admin(self):
        self.setUpLoggedInUser(is_superuser=False)
        another_user = self._get_random_user()

        community = self._get_random_community()
        me = self._get_community_member(user=self.user, community=community, is_admin=True)
        member = self._get_community_member(user=another_user, community=community, is_admin=False)

        response = self.client.delete('/community-members/{}/'.format(me.pk))
        self.assertEqual(response.status_code, 403)

    # regular users can remove themselves
    def test_delete_self(self):
        self.setUpLoggedInUser(is_superuser=True)
        another_user = self._get_random_user()

        community = self._get_random_community()
        me = self._get_community_member(user=self.user, community=community, is_admin=False)
        member = self._get_community_member(user=another_user, community=community, is_admin=True)

        response = self.client.delete('/community-members/{}/'.format(me.pk))
        self.assertEqual(response.status_code, 204)



from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth import get_user_model
from djangoldp.models import Model
from djangoldp_community.models import Community, CommunityMember

class Command(BaseCommand):
  help = 'Create a community and append all local users to it'

  def add_arguments(self, parser):
    parser.add_argument('--name', type=str, default="Community", help='Name of your community')

  def handle(self, *args, **options):
    community = Community.objects.get_or_create(name=options['name'])
    for user in get_user_model().objects.all():
      if(not Model.is_external(user)):
        CommunityMember.objects.get_or_create(community=community[0], user=user)

    self.stdout.write(self.style.SUCCESS('Successful created community'))



from django.conf.urls import url
from .views import OpenCommunitiesViewset
from .models import Community
from djangoldp.models import Model

urlpatterns = [
    url(r'^open-communities/', OpenCommunitiesViewset.urls(model_prefix="open-communities",
        model=Community,
        lookup_field=Model.get_meta(Community, 'lookup_field', 'pk'),
        permission_classes=Model.get_meta(Community, 'permission_classes', []),
        fields=Model.get_meta(Community, 'serializer_fields', []),
        nested_fields=Model.get_meta(Community, 'nested_fields', [])))
]



# Generated by Django 2.2.18 on 2021-02-18 10:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_community', '0002_auto_20210217_1205'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='community',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control'], 'ordering': ['slug'], 'verbose_name': 'community', 'verbose_name_plural': 'communities'},
        ),
        migrations.AlterModelOptions(
            name='communitycircle',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control'], 'verbose_name': 'community circle', 'verbose_name_plural': 'community circles'},
        ),
        migrations.AlterModelOptions(
            name='communityjoboffer',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control'], 'verbose_name': 'community job offer', 'verbose_name_plural': 'community job offers'},
        ),
        migrations.AlterModelOptions(
            name='communitymember',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control'], 'verbose_name': 'community member', 'verbose_name_plural': 'community members'},
        ),
        migrations.AlterModelOptions(
            name='communityproject',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control'], 'verbose_name': 'community project', 'verbose_name_plural': 'community projects'},
        ),
        migrations.AlterField(
            model_name='community',
            name='name',
            field=models.CharField(blank=True, help_text="Changing a community's name is highly discouraged", max_length=255),
        ),
    ]
